@@ -11,7 +11,8 @@ our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(forkmap);
 
 our $MAX_PROC = 4;
-our $VERSION = '0.2005';
+our $VERSION = '0.2007';
+our $TIMEOUT = 0;
 
 sub new {
   my $class = shift;
@@ -25,6 +26,7 @@ sub _init {
   $self->{max_kids} //= 4;
   $self->{ipc} //= 0;
   $self->{non_blocking} //= 1;
+  $self->{timeout} //= 0;
 }
 
 sub max_kids {
@@ -45,12 +47,19 @@ sub ipc {
   $self->{ipc} = $n;
 }
 
+sub timeout {
+  my ($self, $n) = @_;
+  $n // return $self->{timeout};
+  $self->{timeout} = $n;
+}
+
 sub fmap {
   my ($self, $code) = (shift, shift);
   my %pids = ();
   my @rs = ();  # result set of child return values
   my $max = $self->max_kids;
   my $ipc = $self->ipc;
+  my $timeout = $self->timeout;
   my $non_blocking = $self->non_blocking;
 
   for my $proc (@_) {
@@ -65,7 +74,15 @@ sub fmap {
         }
       }
     }
-
+    my $fn = sub {
+      my $rs = shift;
+      if ($ipc) {
+        $pipe->writer();
+        $pipe->autoflush;
+        print $pipe $rs if defined $rs;
+      }
+      return 1;
+    };
     run_fork {  # processes fork here
       parent {
         my $kid = shift;
@@ -83,11 +100,25 @@ sub fmap {
         }
       }
       child {
-        my $rs = $code->($proc);
-        if ($ipc) {
-          $pipe->writer();
-          $pipe->autoflush;
-          print $pipe $rs if defined $rs;
+        if (!$timeout) {
+          my $rs = $code->($proc);
+          $fn->($rs);
+          exit;
+        }
+        eval {
+          local $SIG{ALRM} = sub {
+            die "alarm\n"
+          };
+          alarm $timeout;
+          my $rs = $code->($proc);
+          $fn->($rs);
+          alarm 0;
+        };
+        if ($@) {
+          if ($@ eq "alarm\n") {
+            print STDERR "error: timeout $$\n";
+          }
+          die unless $@ eq "alarm\n";
         }
         exit;
       }
@@ -96,7 +127,6 @@ sub fmap {
       }
     };
   }
-
   1 while (wait() != -1);  # wait for the stragglers to finish
   return @rs;
 }
@@ -120,8 +150,26 @@ sub forkmap (&@) {
         $pids{$kid}++;
       }
       child {
-        local $_ = $proc;
-        $code->();
+        if (!$TIMEOUT) {
+          local $_ = $proc;
+          $code->();
+          exit;
+        }
+        eval {
+          local $SIG{ALRM} = sub {
+            die "alarm\n"
+          };
+          alarm $TIMEOUT;
+          local $_ = $proc;
+          $code->();
+          alarm 0;
+        };
+        if ($@) {
+          if ($@ eq "alarm\n") {
+            print STDERR "error: timeout $$\n";
+          }
+          die unless $@ eq "alarm\n";
+        }
         exit;
       }
       error {
@@ -183,6 +231,22 @@ This module provides mapping with built-in forking.
 
   forkmap { foo($_) } @x;
 
+=head1 VARIABLES
+
+These C<our> variables control only the functional interface.
+
+=head2 MAX_PROC
+
+Max parallelism.
+
+  $Proc::Forkmap::MAX_PROC = 4;
+
+=head2 TIMEOUT
+
+Max time in seconds any single child process can run.
+
+  $Proc::Forkmap::TIMEOUT = 2;
+
 =head1 METHODS
 
 =head2 new
@@ -229,9 +293,11 @@ Turn on/off inter-process communication.
 
 If IPC is on, then set IO::Handle blocking state. This might be useful for conditional parallelism.
 
-=head1 TODO
+=head2 timeout
 
-1. Timeouts
+Timeout in seconds.
+
+  $p->timeout(2);
 
 =head1 SEE ALSO
 
